
; FM SID driver. v1.3 (4mat/ate bit 2016)

; Build mode controls:

; DEBUGMODE        - 0 = build raw player for use elsewhere | 1 = have front-end player built-in.
; DEBUGCRUNCH      - 0 = keep BASIC line | 1 = build without basic line if you're going to crunch it. (exe address will be $0801)

; DEBUGSKIP        - If this is set to 1 pressing SPACE will fast forward your track in front-end mode.
; DEBUGRASTERTIME  - If this is set to 1 the driver rastertime is displayed in front-end.

; DEBUGBORDER      - Colour of border in front-end
; DEBUGPAPER       - And screen color.
; DEBUGRASTERCOLOR - And raster color if you want too.

; DEBUGPATTERNSTART - Skip forwards X song positions before song starts playing.  Handy for testing later parts of your song though obviously it takes a little while to get there.  
		      

; Check fmsid_pictures.asm to set filenames of diplayed ascii picture in front-end.

; NTSC             - If using the built-in SID driver set this to 1 to use an NTSC frequency table.  If using your own SID driver you must set this NTSC in whatever SID editor you use.
;                    As far as I can tell the FM chip automatically switches to NTSC based on the hardware it's plugged into.
                     
; ADDRESS          - Start address of driver. (when building raw, or if using your own sid and you need to move it out of the way)

; OWNSIDPLAYER     - 0 = Use built-in sid player. | 1 = use your own sid driver and only have my FM bit. 
;                    When building with DEBUGMODE set to 0 it only builds my FM bit. 
; OWNSIDADDRESS    - Start address of your own sid driver.  Assumes INIT = JSR OWNSIDADDRESS and PLAY = JSR OWNSIDADDRESS+$03
; OWNSIDSKIPFRAMES - Some recent players use 'ghostbytes' so the SID isn't written until a frame or two after the player is called.  To keep the FM & SID in sync you can
;                    increase this number to call the play routine X times during the init phase to catch up.   Leave as 0 to not call it. 

; Check fmsid_external.asm to set filename of external sid song.

; TEMPO1 / TEMPO2  - These are the tempo values, it'll flip between them on each beat so you can have swing/funk tempo in your song.  Obviously set both the same for solid tempo.  
;                    I don't autoset these in the converter because sometimes songs sound better at different speeds.

NTSC = 0

DEBUGMODE = 1
DEBUGCRUNCH = 0
DEBUGSKIP = 1
DEBUGRASTERTIME = 1
DEBUGBORDER = 14
DEBUGPAPER = 6
DEBUGRASTERCOLOR = 6
DEBUGPATTERNSTART = 0

ADDRESS = $2000

OWNSIDPLAYER = 0
OWNSIDADDRESS = $1000
OWNSIDSKIPFRAMES = 9

TEMPO1 = $06 
TEMPO2 = $08 



	.IF DEBUGMODE = 1
	processor 6502
	org	$0801
	.IF DEBUGCRUNCH = 0
	.byte	$0b,$08,$ef,$00,$9e,$32,$30,$36,$32,0,0,0,0
	org	$080e
	.ENDIF
	
	sei
	
	jsr screensetup

	jsr init



loop
	.IF DEBUGSKIP = 1
	
	lda $dc01
	cmp #$ef
	beq skipraster
	
	.ENDIF
	
	lda $d012
	cmp #$80
	bne loop

skipraster	
	
	.IF DEBUGRASTERTIME = 1
	lda #DEBUGRASTERCOLOR
	sta $d020
	.ENDIF
	jsr playframe
	
	.IF DEBUGRASTERTIME = 1
	lda #DEBUGBORDER
	sta $d020
	.ENDIF
	
	jmp loop
	

	.IF OWNSIDPLAYER = 1
	org OWNSIDADDRESS-2
	.include fmsid_external.asm
	.ENDIF
	
	.ENDIF
	
;======== start of driver


FUNKTEMPO = 1 ; Using different tempos on alternate frames to create a 'funk' rhythm.
VIBRATO = 0 ; This doesn't work anymore so always 0.
SONGREPEAT = 1 ; If you are using the Fx command (repeat next pattern x times command) in song data.

	org ADDRESS
	jmp init
	jmp playframe

init

	lda #$0f
	sta $d418

	
	ldx #$00
clearfm
	txa
	sta $df40
	lda #$00
	sta $df50
	inx
	cpx #$c9
	bne clearfm

	lda #$00
	sta ticks

	.IF OWNSIDPLAYER = 0

	ldy #$01
	ldx #$00
findinstruments	
	lda insttab,x
	cmp #$ef
	bcc notaninst	
	cmp #$ff
	beq instfindexit
	txa
	clc
	adc #$01
	sta instoff,y
	sta $0340,y
	iny
notaninst
	inx
	cpx #$00
	bne findinstruments
instfindexit	
	.ENDIF
	
	ldx #$0b
resetgo
	
	lda songstarthi,x 
	sta songposhi,x
	lda songstartlo,x
	sta songposlo,x
	jsr getpat
	dex
	bpl resetgo
	
	.IF OWNSIDPLAYER = 1
	
	jsr OWNSIDADDRESS


	.IF OWNSIDSKIPFRAMES > 0
	lda #OWNSIDSKIPFRAMES
	sta skipframes
skipframesloop	
	jsr OWNSIDADDRESS+$03
	dec skipframes
	lda skipframes
	cmp #$00
	bne skipframesloop
	.ENDIF
	
	.ENDIF


	.IF DEBUGPATTERNSTART > 0
	
	lda #65
	sta 53265
	
	lda #$34
	sta $01
	
	lda songposlo+$03
	sta temppatternstart
	lda #DEBUGPATTERNSTART
	sta temppatternamount

skipforwards
	jsr playframe
	
	lda songposlo+$03
	cmp temppatternstart
	beq skipforwards
	sta temppatternstart
	dec temppatternamount
	lda temppatternamount
	cmp #$00
	bne skipforwards
	
	
	lda #$35
	sta $01	
	lda #155
	sta 53265

	
	.ENDIF
	
	rts       

skipframes
	.byte $00

	.IF DEBUGPATTERNSTART > 0
temppatternstart
	.byte $00
temppatternamount
	.byte $00
	.ENDIF

	
;;; playframe


playframe

	.IF OWNSIDPLAYER = 0
	ldx #$00
	lda #$00 
	.ELSE
	ldx #$03
	lda #$10
	.ENDIF

	sta temp2

noteloop
	stx temp
	lda ticks
	beq dothis
	jmp update

dothis
	dec delay,x
	bmi newnote
	jmp update

newnote
	lda transtemp,x
	sta transpose,x

	jsr getnewnote
	
nonewpat
	cmp #$ef
	bcc justnote
instrument
	jsr pataddwork
	jsr getnewnote
	sta instnum,x
fubme
	jsr pataddwork
	;inc pattpos,x
	jmp newnote

justnote
	bpl regularnote
	jsr pataddwork
	jsr getnewnote
	sta notelen,x 
	jmp fubme

regularnote
	sta tempnote
	ldy instnum,x
	lda notelen,x
	sta delay,x
	;inc pattpos,x
	jsr pataddwork
	;pla	
	lda tempnote
	cmp #$00
	beq update
	sta note,x

doit

	lda instoff,y
	sta yahoo,x
	

	lda resetwav,x
	cmp instnum,x
	bne havetoreset

	lda instdec,y
	and #$f0
	bne nowavreset

havetoreset

	lda instpul,y
	and #$0f
	sta insthigh,x
	lda #$00
	sta instfreq,x

nowavreset

	lda instfilt,y
	beq nofilt
	sta filtset
	sty filtnum
	lda filtres,y
	sta $d417
	lda volume,y
	sta $d418
nofilt
	tya
	sta resetwav,x


	ldx temp2
	cpx #$0f
	bcc itssid
	
	
	.IF DEBUGPATTERNSTART = 0
	jsr setfm
	.ELSE
	lda temppatternamount
	cmp #$00
	bne skipfm
	jsr setfm		
skipfm		
	.ENDIF
	
	
	jmp update
	
	
itssid

	

	lda instsus,y
	sta $d406,x
	lda instadc,y
	sta $d405,x
	lda #$09
	sta $d404,x
	
update

	ldx temp

	lda instnum,x
	sta temp3



	ldy yahoo,x
	lda instnote,y
	ldy temp2
	cmp #$00
	beq playusual
	cmp #$f0
	bcc playthis
	and #$0f
	bpl playusual
playthis
	sta temp5
	lda #$00
	sta temp4
	jmp playcont

playusual
	clc
	adc note,x
	adc transpose,x

playnowt
	tax
	lda freqtablo,x
	sta temp5
	lda freqtabhi,x
	sta temp4

; hum hum

	ldx temp
	ldy temp3
	lda instdec,y
	and #$01
	beq nowave
	lda temp5
	adc instfreq,x
	sta temp5

nowave

	.IF VIBRATO = 1
	ldy temp3
	lda instpul,y
	.ENDIF
	
	ldy temp2
	
	.IF VIBRATO = 1
	cmp #$7f
	bcc playcont

;;;;;;;;;;;;;;; not implemented

	.ENDIF
	
	
playcont

	cpy #$0f
	bcc itssid2
	
	jmp nogate
	
itssid2
	lda temp5
	sta $d401,y
	lda temp4
	sta $d400,y
	;ldy temp2 ; add


	;ldx temp
	lda instfreq,x
	sta $d402,y
	lda insthigh,x
	sta $d403,y

	lda yahoo,x
	tax
	lda insttab,x
	sta $d404,y

nogate
	
	ldx temp


	ldy temp3
	;clc
	lda instfreq,x
	adc instadd,y 
	sta instfreq,x
	lda instfreq,x
	cmp instadd,y
	bcs noise
	inc insthigh,x
	


noise 


	inc yahoo,x
	ldy yahoo,x
	;tay
	lda insttab,y
	cmp #$ef
	bcc noinstres
	and #$0f
	sta temp
	lda yahoo,x
	sbc temp
	sta yahoo,x
	
	
	
noinstres
	clc
	lda temp2
	adc #$07
	sta temp2

	jsr getnewnote
	
	;lda pattposhi,x
	;sta readpat2+$02
	;lda pattposlo,x
	;sta readpat2+$01

;readpat2
;	lda $4000
	;ldy pattpos,x
	;tay
	;lda patttab,y
	cmp #$7f
	bne suxx
	
	;; put getpat here
	jsr getpat
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;	rts

	;jmp noteloop
suxx

	inx
	cpx #$0c ;3

	;dec notewait,x

	beq updatend
	jmp noteloop ;updateloop
updatend

	lda ticks
	;cmp #$00 ; ff
	bne noreset

	.IF FUNKTEMPO = 1
	inc tempocnt
	lda tempocnt
	and #$01
	sta tempocnt
	tax
	lda tempo,x
	
	.ELSE
	
	lda tempo

	.ENDIF
	
	
	;ldx tempocnt
	;cpx #$02
	;bne resetnor
	;ldx #$00
	;stx tempocnt
	;lda tempo,x
	sta ticks 

noreset
	dec ticks

	clc
	ldy filtnum
	lda filtset
	adc instfila,y
	sta filtset
	sta $d416

	.IF OWNSIDPLAYER = 1
	jsr OWNSIDADDRESS+$03
	.ENDIF
	
	rts

getnewnote
	lda pattposhi,x
	sta readpat+$02
	lda pattposlo,x
	sta readpat+$01
readpat
	lda $4000
	rts
	
pataddwork
	
	inc pattposlo,x
	lda pattposlo,x
	cmp #$00
	bne patnowork
	inc pattposhi,x	
patnowork
	rts

getpat

	lda songposhi,x
	sta readsong0+$02
	lda songposlo,x
	sta readsong0+$01
	
readsong0 
	lda $4000

	;ldy songpos,x
	;tay

	;lda songstarthi,x 
	;sta readsong1+$02
	;lda songstartlo,x
	;sta readsong1+$01
;readsong1
;	lda $4000,y ;songtab,y
	cmp #$ff
	bne nopatreset

	lda songstarthi,x 
	sta songposhi,x
	lda songstartlo,x
	sta songposlo,x
;setsong2	
;	lda $4000 ;songstart,x
;	sta songpos,x
	jmp getpat

nopatreset 

	; added
	.IF SONGREPEAT = 1
	cmp #$df
	bcc nopatloop
	
yesrepeat
	and #$0f
	sta patwait,x
	jsr incsongpos
	;inc songpos,x
	jmp getpat
	.ENDIF
	
; added

nopatloop
	cmp #$cf
	bcc notranspose
	
	and #$0f
	sta transtemp,x
	jsr incsongpos
	;inc songpos,x
	jmp getpat
	
notranspose
	;sta temp
	;and #$0f
	tay 
	
	lda pattstarthihi,x
	sta getpattoffsethi+$02
	lda pattstarthilo,x
	sta getpattoffsethi+$01
	
getpattoffsethi lda $1000,y
	sta pattposhi,x

	lda pattstartlohi,x
	sta getpattoffsetlo+$02
	lda pattstartlolo,x
	sta getpattoffsetlo+$01
	
getpattoffsetlo lda $1000,y
	sta pattposlo,x
	
	;lda pattstarthi,y
	;sta pattposhi,x
	;lda pattstartlo,y
	;sta pattposlo,x
	
	;lda temp
	;and #$f0
	;jsr rormedo
	;lsr
	;lsr
	;lsr
	;lsr
	;sta transtemp,x  ;transpose,x
	; inc songpos,x
	; rts

	dec patwait,x
	lda patwait,x 
	bpl notfin
	jsr incsongpos
	;inc songpos,x
	lda #$00
	sta patwait,x
notfin
	rts
	
incsongpos
	inc songposlo,x
	lda songposlo,x
	cmp #$00
	bne incsongposexit
	inc songposhi,x
incsongposexit
	rts
	
setfm

	stx tempfm00
	sty tempfm01
	
	ldx temp
	ldy instnum,x
	lda fminstoffsetlo,y
	sta readparam+$01
	lda fminstoffsethi,y
	sta readparam+$02
	
	
	
	lda temp
	clc
	sbc #$02
	asl
	asl
	asl
	asl
	sta tempfm02

	
	tax
	lda openoffsets+$0a,x
	clc
	adc #$b0
	sta $df40

	nop
	nop
	nop
	nop
	lda #$00
	sta $df50
        ldx #4
fmpause1        
	dex
        nop
        bne fmpause1
	
	ldy #$00
setregis
	ldx tempfm02
	lda openoffsets,x
	clc
	adc openchan,y
	tax
readparam	
	lda $4000,y
        stx $df40               
        nop
        nop
        nop
        nop                     
        sta $df50              
        ldx #4
fmpause2
	dex
        nop
        bne fmpause2
        inc tempfm02
	iny
	cpy #$09
	bne setregis
	
	ldy #$09
	ldx tempfm02
	lda openoffsets,x
	clc
	adc openchan,y
	sta $df40
	nop
	nop
	nop
	nop
	ldx temp
	lda note,x
	clc
	adc transpose,x
	tay
	lda fmfreqlo,y
	sta $df50

	ldy #$0a	
	inc tempfm02
	ldx tempfm02
	lda openoffsets,x
	clc
	adc openchan,y
	sta $df40
	nop
	nop
	nop
	nop
	ldx temp
	lda note,x
	clc
	adc transpose,x
	tay
	lda fmfreqhi,y
	sta $df50
	

	ldx tempfm00
	ldy tempfm01
	
	rts	

tempfm00
	.byte $00
tempfm01
	.byte $00
tempfm02
	.byte $00
	
openchan
	.byte $20,$20,$40,$40,$60,$60,$80,$80,$c0,$a0,$b0
	
openoffsets

	.byte $00,$03,$00,$03,$00,$03,$00,$03,$00,$00,$00,$ff,$ff,$ff,$ff,$ff
	.byte $01,$04,$01,$04,$01,$04,$01,$04,$01,$01,$01,$ff,$ff,$ff,$ff,$ff
	.byte $02,$05,$02,$05,$02,$05,$02,$05,$02,$02,$02,$ff,$ff,$ff,$ff,$ff
	.byte $08,$0b,$08,$0b,$08,$0b,$08,$0b,$03,$03,$03,$ff,$ff,$ff,$ff,$ff
	.byte $09,$0c,$09,$0c,$09,$0c,$09,$0c,$04,$04,$04,$ff,$ff,$ff,$ff,$ff
	.byte $0a,$0d,$0a,$0d,$0a,$0d,$0a,$0d,$05,$05,$05,$ff,$ff,$ff,$ff,$ff
	.byte $10,$13,$10,$13,$10,$13,$10,$13,$06,$06,$06,$ff,$ff,$ff,$ff,$ff
	.byte $11,$14,$11,$14,$11,$14,$11,$14,$07,$07,$07,$ff,$ff,$ff,$ff,$ff
	.byte $12,$15,$12,$15,$12,$15,$12,$15,$08,$08,$08,$ff,$ff,$ff,$ff,$ff

fmfreqhi
	.byte $00 ;$00,$00
	.byte $21,$21,$21,$21,$21,$21,$21,$22,$22,$22,$22,$22
	.byte $22,$25,$25,$25,$25,$25,$25,$26,$26,$26,$26,$26	
	.byte $26,$29,$29,$29,$29,$29,$29,$2a,$2a,$2a,$2a,$2a
	.byte $2a,$2d,$2d,$2d,$2d,$2d,$2d,$2e,$2e,$2e,$2e,$2e	
	.byte $2e,$31,$31,$31,$31,$31,$31,$32,$32,$32,$32,$32	
	.byte $32,$35,$35,$35,$35,$35,$35,$36,$36,$36,$36,$36	
	.byte $36,$39,$39,$39,$39,$39,$39,$3a,$3a,$3a,$3a,$3a
	.byte $3a,$3d,$3d,$3d,$3d,$3d,$3d,$3e,$3e,$3e,$3e,$3e	
	
fmfreqlo
	.byte $00 ;$00,$00
	.byte $ae,$6b,$81,$98,$b0,$ca,$e5,$02,$20,$41,$63,$87
	.byte $ae,$6b,$81,$98,$b0,$ca,$e5,$02,$20,$41,$63,$87
	.byte $ae,$6b,$81,$98,$b0,$ca,$e5,$02,$20,$41,$63,$87
	.byte $ae,$6b,$81,$98,$b0,$ca,$e5,$02,$20,$41,$63,$87
	.byte $ae,$6b,$81,$98,$b0,$ca,$e5,$02,$20,$41,$63,$87
	.byte $ae,$6b,$81,$98,$b0,$ca,$e5,$02,$20,$41,$63,$87
	.byte $ae,$6b,$81,$98,$b0,$ca,$e5,$02,$20,$41,$63,$87
	.byte $ae,$6b,$81,$98,$b0,$ca,$e5,$02,$20,$41,$63,$87
	

	.include fmsid_fminst.asm
	
 	.include fmsid_sidinst.asm

         ; songdata description:
         ; songstart : songtab offset for each channel
         ; songtab   : xy = transpose/pattern , ex = rewind x , fx = repeat x , ff = end  
                 
tempo .byte TEMPO1,TEMPO2 

songstartlo 

	.byte #<song0,#<song1,#<song2,#<song3,#<song4,#<song5,#<song6,#<song7,#<song8,#<song9,#<song10,#<song11

songstarthi 

	.byte #>song0,#>song1,#>song2,#>song3,#>song4,#>song5,#>song6,#>song7,#>song8,#>song9,#>song10,#>song11


       
         ; pattstart : patttab offset for each pattern 
         ; patttab   : 8x = set note duration, fx = set instrument,$ex = rewind x, 7f = end  

pattstartlohi
	.byte #>pattstartlo0,#>pattstartlo1,#>pattstartlo2,#>pattstartlo3,#>pattstartlo4,#>pattstartlo5,#>pattstartlo6,#>pattstartlo7,#>pattstartlo8,#>pattstartlo9,#>pattstartlo10,#>pattstartlo11
pattstartlolo
	.byte #<pattstartlo0,#<pattstartlo1,#<pattstartlo2,#<pattstartlo3,#<pattstartlo4,#<pattstartlo5,#<pattstartlo6,#<pattstartlo7,#<pattstartlo8,#<pattstartlo9,#<pattstartlo10,#<pattstartlo11
pattstarthihi 
	.byte #>pattstarthi0,#>pattstarthi1,#>pattstarthi2,#>pattstarthi3,#>pattstarthi4,#>pattstarthi5,#>pattstarthi6,#>pattstarthi7,#>pattstarthi8,#>pattstarthi9,#>pattstarthi10,#>pattstarthi11
pattstarthilo
	.byte #<pattstarthi0,#<pattstarthi1,#<pattstarthi2,#<pattstarthi3,#<pattstarthi4,#<pattstarthi5,#<pattstarthi6,#<pattstarthi7,#<pattstarthi8,#<pattstarthi9,#<pattstarthi10,#<pattstarthi11
	
	.include musdata.asm
         
		; frequency table by MrSid from codebase64. (better than my generated one)

	.IF NTSC = 0
	
freqtabhi
		.byte $00
	        ;      C   C#  D   D#  E   F   F#  G   G#  A   A#  B
                .byte $17,$27,$39,$4b,$5f,$74,$8a,$a1,$ba,$d4,$f0,$0e  ; 1
                .byte $2d,$4e,$71,$96,$be,$e8,$14,$43,$74,$a9,$e1,$1c  ; 2
                .byte $5a,$9c,$e2,$2d,$7c,$cf,$28,$85,$e8,$52,$c1,$37  ; 3
                .byte $b4,$39,$c5,$5a,$f7,$9e,$4f,$0a,$d1,$a3,$82,$6e  ; 4
                .byte $68,$71,$8a,$b3,$ee,$3c,$9e,$15,$a2,$46,$04,$dc  ; 5
                .byte $d0,$e2,$14,$67,$dd,$79,$3c,$29,$44,$8d,$08,$b8  ; 6
                .byte $a1,$c5,$28,$cd,$ba,$f1,$78,$53,$87,$1a,$10,$71  ; 7
                .byte $42,$89,$4f,$9b,$74,$e2,$f0,$a6,$0e,$33,$20,$ff  ; 8

freqtablo
		.byte $00
		;      C   C#  D   D#  E   F   F#  G   G#  A   A#  B
                .byte $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$02  ; 1
                .byte $02,$02,$02,$02,$02,$02,$03,$03,$03,$03,$03,$04  ; 2
                .byte $04,$04,$04,$05,$05,$05,$06,$06,$06,$07,$07,$08  ; 3
                .byte $08,$09,$09,$0a,$0a,$0b,$0c,$0d,$0d,$0e,$0f,$10  ; 4
                .byte $11,$12,$13,$14,$15,$17,$18,$1a,$1b,$1d,$1f,$20  ; 5
                .byte $22,$24,$27,$29,$2b,$2e,$31,$34,$37,$3a,$3e,$41  ; 6
                .byte $45,$49,$4e,$52,$57,$5c,$62,$68,$6e,$75,$7c,$83  ; 7
                .byte $8b,$93,$9c,$a5,$af,$b9,$c4,$d0,$dd,$ea,$f8,$ff  ; 8

	.ELSE
	
freqtabhi
		.byte $00
		;      C   C#  D   D#  E   F   F#  G   G#  A   A#  B
                .byte $0c,$1c,$2d,$3f,$52,$66,$7b,$92,$aa,$c3,$de,$fa  ; 1
                .byte $18,$38,$5a,$7e,$a4,$cc,$f7,$24,$54,$86,$bc,$f5  ; 2
                .byte $31,$71,$b5,$fc,$48,$98,$ee,$48,$a9,$0d,$79,$ea  ; 3
                .byte $62,$e2,$6a,$f8,$90,$30,$dc,$90,$52,$1a,$f2,$d4  ; 4
                .byte $c4,$c4,$d4,$f0,$20,$60,$b8,$20,$a4,$34,$e4,$a8  ; 5
                .byte $88,$88,$a8,$e0,$40,$c0,$70,$40,$48,$68,$c8,$50  ; 6
                .byte $10,$10,$50,$c0,$80,$80,$e0,$80,$90,$d0,$90,$a0  ; 7
                .byte $20,$20,$a0,$80,$00,$00,$c0,$00,$20,$a0,$20,$40  ; 8
     
freqtablo
		.byte $00
		;      C   C#  D   D#  E   F   F#  G   G#  A   A#  B
                .byte $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01  ; 1
                .byte $02,$02,$02,$02,$02,$02,$02,$03,$03,$03,$03,$03  ; 2
                .byte $04,$04,$04,$04,$05,$05,$05,$06,$06,$07,$07,$07  ; 3
                .byte $08,$08,$09,$09,$0a,$0b,$0b,$0c,$0d,$0e,$0e,$0f  ; 4
                .byte $10,$11,$12,$13,$15,$16,$17,$19,$1a,$1c,$1d,$1f  ; 5
                .byte $21,$23,$25,$27,$2a,$2c,$2f,$32,$35,$38,$3b,$3f  ; 6
                .byte $43,$47,$4b,$4f,$54,$59,$5e,$64,$6a,$70,$77,$7e  ; 7
                .byte $86,$8e,$96,$9f,$a9,$b3,$bd,$c9,$d5,$e1,$ef,$fd  ; 8	
	.ENDIF

songposhi .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
songposlo .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
pattposhi .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
pattposlo .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
transpose .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
note .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
instnum .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
delay .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
repeaty .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
yahoo .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
instfreq .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
insthigh .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
notelen .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
patwait .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
resetwav .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
patrepeat .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
transtemp .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
filtset .byte $00
ticks .byte $00
tempocnt .byte $00
insvibc .byte $00
insvibd .byte $00
insvibz .byte $00
filtnum .byte $00
temp .byte $00
temp2 .byte $00
temp3 .byte $00
temp4 .byte $00
temp5 .byte $00	
tempnote .byte $00

	.IF DEBUGMODE = 1

screensetup
	ldx #$00
clearscreen
	lda petsciicol+$02,x
	sta $d800,x
	lda petsciicol+$102,x
	sta $d900,x
	lda petsciicol+$202,x
	sta $da00,x
	lda petsciicol+$302,x
	sta $db00,x
	lda petsciigfx+$02,x
	sta $0400,x
	lda petsciigfx+$102,x
	sta $0500,x
	lda petsciigfx+$202,x
	sta $0600,x
	lda petsciigfx+$302,x
	sta $0700,x
	inx
	bne clearscreen
	
	lda #21
	sta 53272
	lda #DEBUGPAPER
	sta $d021
	lda #DEBUGBORDER
	sta $d020

	rts

	.include fmsid_pictures.asm

	.ENDIF

	
	
	